clear all
close all

set(0, 'DefaultFigureRenderer', 'painters');

fsep = filesep;

folder = ['preprocessed' fsep]

% %loading data with no pbs from March, 11th for comparison
% load([folder, 'all_2021_03_11_220540.mat'])
% out_noPBS = out;
% clear out


% ion 1, 2
files = {'2021_03_20set2_MemRamTr_0_l_1to190.mat', '2021_03_20set2_MemRamTr_3_l_1to190.mat'}
standardbell;
rho_base{1} = phipm; rho_base{3} = phimm; rho_base{2} = psipm; rho_base{4} = psimm; %define Bell states

%%  reconstruct 8 photon-photon states and find rotations
prefix = ['photon_photon_individual' fsep 'swapped_statesCn_'];
clear out out1 out2

% ION 1
%below use the last parameter 500 instead of 50 to have Mc = 500 Monte_carlo
%states as in the paper. 50 used for faster processing now
doTomo_cumulative_swap_condv2([folder fsep files{1}], [folder fsep prefix files{1}], 50); %use to reevaluate, comment to use saved ones
load([folder prefix files{1}]) % load 4 reconstructed states ion 1 first
out1 = out; %First in time
U1 = find_opt_U4(out1.rhoML, rho_base);  %get unitary to rotate to Bell states
clear out

% ION 2
doTomo_cumulative_swap_condv2([folder fsep files{2}], [folder fsep prefix files{2}], 50); %use to reevaluate, comment to use saved ones
load([folder, prefix, files{2}]) % 4 reconstructed states ion 2 first
out2 = out; %Second in time
U2 = find_opt_U4(out2.rhoML, rho_base);  %get unitary to rotate to Bell states

%save([folder  'U1'], 'U1');save([folder  'U2'], 'U2')
%load([folder  'U1']);  load([folder  'U2']);
%%[U1,U2] = find_opt_U8(out1.rhoML,out2.rhoML, rho_base);
%%load([folder  'U1_model']);  load([folder  'U2_model']);

%% plotting and normalizing wavepackets
fw2 = figure();
fw1 = figure();
dt = 1;
bins =  [1:dt:250];
countsA = sum(hist(out1.Counts1{1}, bins),2);
countsA_loop1 = hist(out1.WP_phot_loop1{1}, bins);
countsA_loop2 = hist(out1.WP_phot_loop2{1}, bins);
countsB = sum(hist(out2.Counts1{1}, bins),2);
countsB_loop1 = hist(out2.WP_phot_loop1{1}, bins);
countsB_loop2 = hist(out2.WP_phot_loop2{1}, bins);



start = fix(out.gatestart{1});
stop = fix(out.gatestop{1});%+5;


attemptsA = 0;
attemptsB = 0;
attemptsA_L1 = 0;
attemptsA_L2 = 0;
attemptsB_L1 = 0;
attemptsB_L2 = 0;
all_mem_loop_attempts_hist = zeros(200,1)'; %
for i = 1:36
    %ion B
    attemptsB = attemptsB +sum(out2.nloops{1}{i});% all attempts. old calculation
    meml = find(out1.nloops2{1}{i}<=out1.nloops{1}{i});% loops where B was memory
    nonmeml = find(out1.nloops2{1}{i}>out1.nloops{1}{i});%loops where B was not memory
    attemptsB_L1 = attemptsB_L1+sum(out1.nloops2{1}{i}(meml))+sum(out1.nloops{1}{i}(nonmeml)); %sum of number of loops executed whan B was memory and first loops when B was not mem
    attemptsB_L2 = attemptsB_L2+sum(out1.nloops2{1}{i}(nonmeml) - out1.nloops{1}{i}(nonmeml)); %number of second loops when B was memory
    
    %ion A. Copied from ion B and changing out1 to out2 but could be simplified. e.g. AttemptsB_L1 =
    %AttemptsA_L1 
    attemptsA = attemptsA +sum(out1.nloops{1}{i}); 
    meml = find(out2.nloops2{1}{i}<=out2.nloops{1}{i});  
    nonmeml = find(out2.nloops2{1}{i}>out2.nloops{1}{i}); 
    attemptsA_L1 = attemptsA_L1+sum(out2.nloops2{1}{i}(meml))+sum(out2.nloops{1}{i}(nonmeml));
    attemptsA_L2 = attemptsA_L2+sum(out2.nloops2{1}{i}(nonmeml) - out2.nloops{1}{i}(nonmeml));
   
    all_mem_loop_attempts_hist = all_mem_loop_attempts_hist +hist(abs(out1.nloops{1}{i}-out1.nloops2{1}{i}) ,[0:199]);
   
    
end
% all_mem_loop_attempts_hist(1:end-1) =  all_mem_loop_attempts_hist(2:end); all_mem_loop_attempts_hist(end) = 0; %bringing array element 1 memloops 1 instead of memloops 0 

  
%getting cumulative probabilities
for i = start:stop
   xval(i-start+1) = i; 
   cum_wp1(i-start+1) = sum(countsA(start:i))/ attemptsA;
   cum_wp1_loop1(i-start+1) = sum(countsA_loop1(start:i))/ attemptsA_L1;
   cum_wp1_loop2(i-start+1) = sum(countsA_loop2(start:i))/ attemptsA_L2;
   
   cum_wp2(i-start+1) = sum(countsB(start:i))/ attemptsB;
   cum_wp2_loop1(i-start+1) = sum(countsB_loop1(start:i))/ attemptsB_L1;
   cum_wp2_loop2(i-start+1) = sum(countsB_loop2(start:i))/ attemptsB_L2;
end
%% additional delays calculation. see sequence file
aod_delay = 5;
raman_length = 50;
delay_between = 5;
fiber_time = 25.25e3/(2.998e8/1.4682);

del = aod_delay; % 5 us  delay between trigger and actual Raman pulse see sequence
loop1_hardw_gate(1) = (125-2*(raman_length+aod_delay)-delay_between-5)+2*(raman_length+aod_delay)+delay_between+0.3-del; %loop1 logic gates phot1
loop1_hardw_gate(2) = loop1_hardw_gate(1) +raman_length+5;
loop1_hardw_gate(3) = loop1_hardw_gate(2) +delay_between-raman_length-delay_between-0.1-del; %loop1 logic gates phot2
loop1_hardw_gate(4) = loop1_hardw_gate(3) +raman_length+10;
loop2_hardw_gate(1) = (125-(1*raman_length+aod_delay)-14)+raman_length+aod_delay+2-del;  %loop2 logic gates
loop2_hardw_gate(2) = loop2_hardw_gate(1) +raman_length+10;


%% plotting panel a (Wavepackets)
set(0,'defaultTextInterpreter','latex');
set(0, 'defaultAxesTickLabelInterpreter','latex'); set(0, 'defaultLegendInterpreter','latex');
main_figure = figure('DefaultAxesFontSize',22);


%Wavepacket A
subplot(1,2,1)
pbaspect([2 1 1])
yyaxis left
hold on

bl = bar(bins-del, countsA_loop1/attemptsA_L1*dt, 'BarWidth', 1, 'EdgeColor', 'none', 'FaceAlpha', 0.4,'FaceColor', 'r' );
bl = bar(bins-del, countsA_loop2/attemptsA_L2*dt, 'BarWidth', 1, 'EdgeColor', 'none', 'FaceAlpha', 0.5,'FaceColor', 'r' );
%plot(bins-del, countsA, '--', 'Color', 'b', 'LineWidth', 2 );  %average over loop1 loop2  
title('Alice', 'Color', 'r')
xlim([120,200]-del-3)
ylim([0,2e-4])
% xline(start,'--k') %not supported in matlab 2016
% xline(stop,'--k')
a = gca;
yl = a.YLim;
plot([start,start]-del,yl, '--','Linewidth', 1.5, 'color', 'black')
plot([stop,stop]-del, yl, '--','Linewidth', 1.5,'color', 'black')
%shading for hardware gate
% t = fill([loop1_hardw_gate(1:2), loop1_hardw_gate(2:-1:1)]', [yl(1), yl(1), yl(2), yl(2)]', 'r');
% set(t, 'FaceAlpha', 0.1, 'LineStyle', 'none', 'FaceColor', [0.4,0.2,0.2])
% t = fill([loop2_hardw_gate(1:2), loop2_hardw_gate(2:-1:1)]', [yl(1), yl(1), yl(2), yl(2)]', 'r');
% set(t, 'FaceAlpha', 0.1, 'LineStyle', 'none', 'FaceColor', [0.2,0.4,0.2])
grid on
xlabel('Time ($\mu$s)')
set(gca,  'YColor', bl.FaceColor)
ylabel('$P_d\, (10^{-4}$)', 'Color', 'r')


yyaxis right
hold on
cum_color_L1 = [0.5 0.5 0.5];
cum_color = 'black'
%plot( xval-del, cum_wp1, '--', 'Linewidth', 2, 'Color', cum_color  ); %average over loop1 loop2  

%plot( xval-del, cum_wp1_loop1,'-', 'Linewidth', 2, 'Color', cum_color_L1 ); 
s = shadedErrorBar( xval-del, cum_wp1_loop1, sqrt(cum_wp1_loop1/attemptsA_L1));
set(s.mainLine, 'Color', cum_color_L1, 'LineWidth', 1.5)
set( s.edge, 'LineStyle', 'none')
%plot( xval-del, cum_wp1_loop2, '-','Linewidth', 2, 'Color', cum_color  ); 
s = shadedErrorBar( xval-del, cum_wp1_loop2, sqrt(cum_wp1_loop2/attemptsA_L2));
set(s.mainLine, 'Color', cum_color, 'LineWidth', 1.5)
set( s.edge, 'LineStyle', 'none')
ylim([0,4e-3])
ylabel('Prob\, $(10^{-3})$','Interpreter','latex')
set(gca,  'YColor', cum_color)

%Wavepacket B
%fw2 = figure();
ax2 = subplot(1,2,2);
pbaspect([2 1 1])
yyaxis left
hold on
bl = bar(bins-del, countsB_loop1/attemptsB_L1*dt, 'BarWidth', 1, 'EdgeColor', 'none', 'FaceAlpha', 0.4,'FaceColor', 'b' );
bl = bar(bins-del, countsB_loop2/attemptsB_L2*dt, 'BarWidth', 1, 'EdgeColor', 'none', 'FaceAlpha', 0.5,'FaceColor', 'b' );
%plot(bins-del, countsB, '--', 'Color', 'r', 'LineWidth', 2 );
xlim([120,200]-del-3)
ylim([0,2e-4])
title('Bob', 'Color', 'b')
% xline(start,'--k')
% xline(stop,'--k')
a = gca;
plot([start,start]-del,a.YLim, '--','Linewidth', 1.5, 'color', 'black')
plot([stop,stop]-del, a.YLim, '--','Linewidth', 1.5,'color', 'black')
% t = fill([loop1_hardw_gate(3:4), loop1_hardw_gate(4:-1:3)]', [yl(1), yl(1), yl(2), yl(2)]', 'r');
% set(t, 'FaceAlpha', 0.1, 'LineStyle', 'none', 'FaceColor', [0.4,0.2,0.2])
% t = fill([loop2_hardw_gate(1:2), loop2_hardw_gate(2:-1:1)]', [yl(1), yl(1), yl(2), yl(2)]', 'r');
% set(t, 'FaceAlpha', 0.1, 'LineStyle', 'none', 'FaceColor', [0.2,0.4,0.2])

grid on
xlabel('Time ($\mu$s)')
ylabel('$P_d\,(10^{-4}$)', 'Color', 'b')
set(gca,  'YColor', bl.FaceColor)

yyaxis right
hold on
%plot(ax2, xval-del, cum_wp2,'--', 'Linewidth', 2  , 'Color', cum_color);   %average over loop1 loop2      
%plot(ax2, xval-del, cum_wp2_loop1,'-', 'Linewidth', 2, 'Color', cum_color_L1  ); 
s = shadedErrorBar( xval-del, cum_wp2_loop1, sqrt(cum_wp2_loop1/attemptsB_L1));
set(s.mainLine, 'Color', cum_color_L1, 'LineWidth', 1.5)
set( s.edge, 'LineStyle', 'none')
%plot(ax2, xval-del, cum_wp2_loop2, '-','Linewidth', 2, 'Color', cum_color  );
s = shadedErrorBar( xval-del, cum_wp2_loop2, sqrt(cum_wp2_loop2/attemptsB_L2));
set(s.mainLine, 'Color', cum_color, 'LineWidth', 1.5)
set( s.edge, 'LineStyle', 'none')
ylim([0,4e-3])
set(gca,  'YColor', cum_color)
ylabel('Prob\, $(10^{-3})$')

%% plotting plotting panel b (memsteps distribution)

av_points = 5;
x_val = 2:av_points:(length(out.count_mem_loop2{1})-5);
for i = 1:length(x_val)  % averaging over 5 loops
    data(i) = sum(out.count_mem_loop2{1}(x_val(i):x_val(i)+av_points-1))+sum(out.count_mem_loop{1}(x_val(i):x_val(i)+av_points-1));
    prob_photon_loop2(i) = data(i)/sum(all_mem_loop_attempts_hist(x_val(i):end))/av_points;
    prob_photon_loop2_err(i) = sqrt(data(i))/sum(all_mem_loop_attempts_hist(x_val(i):end))/av_points;
end
fig_memsteps = figure('DefaultAxesFontSize',24)
pbaspect([5 1 1])
%subplot(2,2,[3,4])
yyaxis left
hold on
%prob photon per loop
color_greenish = [0 102 51]/255;
bar(x_val, prob_photon_loop2, 'BarWidth', 1, 'EdgeColor', 'none', 'FaceAlpha', 0.5,'FaceColor', color_greenish ); 
errorbar(x_val([2,37]), prob_photon_loop2([2,37]), prob_photon_loop2_err([2,37]), ...
'Marker', 'none', 'Color', [0,75,25]/255, 'LineWidth',2, 'LineStyle', 'none' ); 
ylim([0, 0.003])
xlim([1, 190])
xlabel('Loop 2 attempt')
ylabel('Prob $(10^{-3})$')
set(gca,  'YColor',color_greenish)

%getting probability per loop
probs = (out.count_mem_loop2{1}+out.count_mem_loop{1})./ ( sum(sum(out1.photon_first{1})) +sum(sum(out2.photon_first{1}))  ) ;
%probs_noPBS = (out_noPBS.count_mem_loop2{1}+out_noPBS.count_mem_loop{1})./ length(find(out_noPBS.nloops2{1}{1}-out_noPBS.nloops{1}{1})  ) ;
% sum(abs(out_noPBS.nloops2{1}{1}-out_noPBS.nloops{1}{1}))
for i = 2:length(find(probs))
   x(i) = i;
   cum_probs(i) = sum(probs(2:i));
 %  cum_probs_noPBS(i) = sum(probs_noPBS(2:i));
   
end
cum_probs_err_max = cum_probs(end)/sqrt(( sum(sum(out1.photon_first{1})) +sum(sum(out2.photon_first{1}))))
yyaxis right
hold on
%cumulative prob
prob_col = 'k'%[0, 129, 255]/255;
%pl = plot( x, cum_probs, 'Linewidth', 2,'Color',prob_col  ); % '#0081ff'  
s = shadedErrorBar( x, cum_probs, sqrt(cum_probs)/sqrt(( sum(sum(out1.photon_first{1})) +sum(sum(out2.photon_first{1}))  )) );
set(s.mainLine, 'Color', prob_col, 'LineWidth', 1.5)
set( s.edge, 'LineStyle', 'none') 
set( s.patch, 'FaceColor', prob_col, 'FaceAlpha', 0.3) 
%pl = plot( x, cum_probs_noPBS,'--', 'Linewidth', 2,'Color', [0, 129, 255]/255 );
xlim([1, 190])
ylim([0, 0.4])

ylabel('Prob')
set(gca,  'YColor', prob_col);
grid minor

%% plotting memsteps distribution version 2 

av_points = 5;
x_val = 2:av_points:(length(out.count_mem_loop2{1})-5);
for i = 1:length(x_val)  % averaging over 5 loops
    data(i) = sum(out.count_mem_loop2{1}(x_val(i):x_val(i)+av_points-1))+sum(out.count_mem_loop{1}(x_val(i):x_val(i)+av_points-1));
    prob_photon_loop2(i) = data(i)/sum(all_mem_loop_attempts_hist(x_val(i):end))/av_points;
    prob_photon_loop2_err(i) = sqrt(data(i))/sum(all_mem_loop_attempts_hist(x_val(i):end))/av_points;
end
fig_memsteps2 = figure('DefaultAxesFontSize',24)

subplot(1,2,1)

%prob photon per loop
color_greenish = [0 102 51]/255;
bar(x_val, prob_photon_loop2, 'BarWidth', 1, 'EdgeColor', 'none', 'FaceAlpha', 0.5,'FaceColor', color_greenish ); 
hold on
errorbar(x_val([2,37]), prob_photon_loop2([2,37]), prob_photon_loop2_err([2,37]), ...
'Marker', 'none', 'Color', [0,75,25]/255, 'LineWidth',2, 'LineStyle', 'none' ); 
ylim([0, 0.003])
xlim([1, 190])
xlabel('Loop 2 attempt')
ylabel('Prob $(10^{-3})$')
set(gca,  'YColor',color_greenish)
grid minor
pbaspect([2.5,1,1])

subplot(1,2,2)
yyaxis left

bar(x_val, prob_photon_loop2*100, 'BarWidth', 1, 'EdgeColor', 'none', 'FaceAlpha', 0.5,'FaceColor', [0, 129, 255]/255 ); 
hold on
errorbar(x_val([2,37]), prob_photon_loop2([2,37])*100, prob_photon_loop2_err([2,37])*100, ...
'Marker', 'none', 'Color', [75,150,169]/255, 'LineWidth',2, 'LineStyle', 'none' ); 

bl = bar(x_val, data, 'BarWidth', 1, 'EdgeColor', 'none', 'FaceAlpha', 0.5,'FaceColor', [0.75, 0.75, 0] );

xlim([1, 190])
xlabel('Attempt')
ylabel('Counts')
set(gca,  'YColor', [0.75, 0.75,0])
pbaspect([2.5,1,1])

%getting probability per loop
probs = (out.count_mem_loop2{1}+out.count_mem_loop{1})./ ( sum(sum(out1.photon_first{1})) +sum(sum(out2.photon_first{1}))  ) ;
%probs_noPBS = (out_noPBS.count_mem_loop2{1}+out_noPBS.count_mem_loop{1})./ length(find(out_noPBS.nloops2{1}{1}-out_noPBS.nloops{1}{1})  ) ;
% sum(abs(out_noPBS.nloops2{1}{1}-out_noPBS.nloops{1}{1}))
for i = 2:length(find(probs))
   x(i) = i;
   cum_probs(i) = sum(probs(2:i));
 %  cum_probs_noPBS(i) = sum(probs_noPBS(2:i));
   
end
cum_probs_err_max = cum_probs(end)/sqrt(( sum(sum(out1.photon_first{1})) +sum(sum(out2.photon_first{1}))))
yyaxis right
hold on
%cumulative prob
prob_col = 'k'%[0, 129, 255]/255;
%pl = plot( x, cum_probs, 'Linewidth', 2,'Color',prob_col  ); % '#0081ff'  
s = shadedErrorBar( x, cum_probs, sqrt(cum_probs)/sqrt(( sum(sum(out1.photon_first{1})) +sum(sum(out2.photon_first{1}))  )) );
set(s.mainLine, 'Color', prob_col, 'LineWidth', 1.5)
set( s.edge, 'LineStyle', 'none') 
set( s.patch, 'FaceColor', prob_col, 'FaceAlpha', 0.3) 
%pl = plot( x, cum_probs_noPBS,'--', 'Linewidth', 2,'Color', [0, 129, 255]/255 );
xlim([1, 190])
ylim([0, 0.4])

ylabel('Prob')
set(gca,  'YColor', prob_col);
grid minor


%% plotting pannel c  (states)
out = out2;

samples = min( out1.samples ,out2.samples);
['samples = ',num2str(samples)]


im = [];
for outc = 1:4
    %rotating states according to Unitary found before to compare with Bell
    %states
    out2.rhoML{outc} = U2*out2.rhoML{outc}*U2';
    out1.rhoML{outc} = U1*out1.rhoML{outc}*U1';
    for sampl = 1:out.samples %and Mc samples
        out2.rhoMC{outc}(:,:,sampl) =U2*out2.rhoMC{outc}(:,:,sampl)*U2';
        out1.rhoMC{outc}(:,:,sampl) =U1*out1.rhoMC{outc}(:,:,sampl)*U1';
    end
    im = [im, max(abs(imag(out2.rhoML{outc})))]; % getting the maximum(absolute) value of all imaginary parts
end
out = out2; 
analyse_tomoresults_cumulative_BELL; % will compare with 4 pre-deifened Bell states here
plot_tomoresults_cumulative_fig_state_marco %plot
out2 = out; 

max_imaginary_part = max(im)
fig_qual(1) = fg_qual;
fig_rho(1) = fg_rho;
%close(figure(7)) % imaginary rho
%close(figure(10)) % F histograms
%close(figure(9)) % C histograms
%close(figure(4)) % wavepackets

%% Get the state with simulated feed-forward
% clear out
% 
% %to recalculate:
out = doTomo_swap_4xN_Usigma_v2({[folder,  files{1}],[folder,  files{2}]}, {U1,U2}, '_Uopt_full', 20);
%use the last parameter to set the number of Mc realizations

% %just to plot:
%load([folder  'Feedforwarded8' filesep 'out_4xN_comb__Uopt_fullCn_' files{1} files{2}])
plot_tomoresults_cumulative1

title('State with simulated feed-forward')
% 
% 
%% % figures formating

 figure(fig_rho);
 
 panel = [4,3,2,1];
 for kk = 1:4 %iterating the pannels
     b = bar3(fig_rho.Children(panel(kk)), rho_base{kk});
     for i = 1:length(b)   b(i).FaceAlpha = '0.0'; end % set to transparent % b(i).LineStyle = ':';  b(i).LineWidth = 1.5; 
 end
% %set(fig_rho(1), 'Position', [129.8000 -35 339.2000 662.4000])
% saveas(fig_rho(1), 'State_separate_outcomes.png')
% %title(sprintf('States of 4 outcomes rotated to Bell states \n Ion 1 in memory cases and ion2 in memory cases added up \n Single U1 used for ion1 mem and single U2 for ion2 mem'), 'FontSize', 12)
% savefig(fig_rho(1), 'State_separate_outcomes')
% %annotation(figure(fig_rho(1)),'TextBox', sprintf('States of 4 outcomes rotated to Bell states \n Ion 1 in memory cases and ion2 in memory cases added up \n Single U1 used for ion1 mem and single U2 for ion2 mem'), 'FontSize', 12)
% 
out = out1; 
analyse_tomoresults_cumulative_BELL; % will compare with 4 pre-deifened Bell states here
out1 = out; 

'Ion A in memory fidelities: Value, +-'
[out1.fidopt', out1.fidoptSTDp',out1.fidoptSTDm']  %fidoptMd
'Ion B in memory fidelities: Value, +-'
[out2.fidopt', out2.fidoptSTDp',out2.fidoptSTDm'] %fidoptMd
