close all
clear all
if ~exist('QOS_path')
        fsep = filesep;
   
    folder = [ 'preprocessed' fsep ];
end
%% nonmemory ion
%Nonmem_Res  = analyse_nonmemory_ion();

%% memory ion
prefix = 'ion_1_l_0to';
postfix = '_reconstructed';
memsteps = [1, 100, 140, 195];
flip_step = 195;
standardbell


for i = 1:length(memsteps)
    file = [folder prefix num2str(memsteps(i)) postfix]
    %eff(i) = efficiency(file);
    load(file) % this is reconstruct
    counts(i) = sum((sum(out.tomo_matrix1_counts{1}))); % total counts
    rhoML{i} = out.rhoML{1};
    fidopt(i) = out.fidopt;
    fidstd(i) = out.fidoptSTD;
    rhoMC{i} = out.rhoMC{1};
    attempts(i) = sum(out.attemptsbases{1});
    eff(i) = counts(i)/attempts(i);
    U{i} = out.optU{1};
    rhoML{i} = out.rhoML{1};
    pur(i) = out.purity;
    pur_Std(i) = out.purSTD;

    if memsteps(i) ==200
        memsteps(i) = 195;
    end

    if memsteps(i) ==flip_step % need to correct odd spin-echos. In the main experiment correction is implemented in the sequqnce. In this one was not.
        T = kron(II,YY); 
        rhoML_rot{i} = T*rhoML{i}*T;
        rhoML_rot{i} = U{1}*rhoML_rot{i}*U{1}';
    else
        rhoML_rot{i} = U{1}*rhoML{i}*U{1}';
    end
    
    for sampl = 1:out.samples
        if memsteps(i) ==flip_step
            rhoMC_rot{i}(:,:,sampl) = U{1}*T*out.rhoMC{1}(:,:,sampl)*T*U{1}'; % need to correct odd spin-echos
        else
            rhoMC_rot{i}(:,:,sampl) = U{1}*out.rhoMC{1}(:,:,sampl)*U{1}';
        end
       fidoptMC_rot(sampl) = mixedfid(rhoMC_rot{i}(:,:,sampl), phipm);
    end
    fidopt_rot(i) = mixedfid(rhoML_rot{i}, phipm); %take the fidelity with phi+ directly!
    fidstd_rot(i) = std(fidoptMC_rot);

end

%% final figure fidelities
loop_duration = 0.33; % 
xrange = [-1, 200];
K = 1;
Fig_Mem = figure(); % try to make nice figure

set(0, 'defaultAxesTickLabelInterpreter','latex');
set(0, 'defaultLegendInterpreter','latex');
fid_fig =  figure('DefaultAxesFontSize',26);%subplot(2,1,2); %Fidelities
s1 = gca;

 hold on
toplot = zeros(max(memsteps),1);
toplot(1) = K*eff(1);

set(s1,  'YColor', 'Black')

errorbar((memsteps+1)*loop_duration, fidopt_rot, fidstd_rot, 'Color' , 'Red', 'LineStyle', 'none', 'Marker', 'd', 'MarkerFaceColor', 'r', 'LineWidth', 2, ...
    'DisplayName', 'ion-photon data','MarkerSize', 15);
xlim(xrange*loop_duration)
ylim([0.59,1])
yticks([0.5 0.6 0.7 0.8 0.9 1])
grid minor

ylabel('Fidelity','Interpreter','latex')
xlabel('Time (ms)','Interpreter','latex')
set(s1,  'YColor', 'Black')




%% fit the ion-photon fidelities
ftg = fittype( 'fun_fid_gauss_fit(x, a)' );
fg = fit( (memsteps+1)'*loop_duration, fidopt_rot', ftg, 'StartPoint', [100] );
Tau = fg.a
bounds = confint(fg, 0.6827);
Tau_plus = bounds(2)
Tau_minus = bounds(1)
xlabel('Loop 2 attempt on ion B ($k$)', 'Interpreter', 'latex')
ylabel('Fidelity')

%% go for interpolatoin and prediction of the swapped state 
x = memsteps;
D = 1;
P1 = 3e-3; %  from exp 
P = D*P1;
X_fine = [1:D:x(end)];
left_ind = 1;
right_ind = 2;


rho_base{1} = phipm; rho_base{3} = phimm; rho_base{2} = psipm; rho_base{4} = psimm;
samples_selection = 10;
fid_iph_plus = zeros(length(X_fine),samples_selection);
fid_iph_minus = zeros(length(X_fine),samples_selection);
fid_swapped_plus = zeros(length(X_fine),samples_selection);
fid_swapped_minus = zeros(length(X_fine),samples_selection);
for point = 1:length(X_fine)  %loop over the interpolated points
    point
    Prob(point) = 1/length(X_fine); % homogenious distribution of nloops executed
    Prob(point) = P*(1-P1)^(D*(point-1));
    
   
   
    rho_interp{point} = gauss_decoh_simple(rhoML_rot{1} ,X_fine(point)*0.33, Tau);
    for sampl = 1:samples_selection % MC samples
        rho_plus = gauss_decoh_simple(rhoMC_rot{1}(:,:,sampl) ,X_fine(point)*0.33, Tau_plus); %mixedfid(rhoMC_rot{1}(:,:,sampl), phipm) %+fidstd_rot(1)
        rho_minus = gauss_decoh_simple(rhoMC_rot{1}(:,:,sampl) ,X_fine(point)*0.33, Tau_minus);%-fidstd_rot(1)
        fid_iph_plus(point,sampl) = mixedfid(rho_plus, phipm);
        fid_iph_minus(point,sampl) = mixedfid(rho_plus, phipm);
        %  rho_interpMC{point} = -(rhoMC_rot{left_ind}*(X_fine(point)-x(right_ind))+rhoMC_rot{right_ind}*(x(left_ind)-X_fine(point)))/(x(right_ind)-x(left_ind));

        fid_fine(point) = mixedfid(rho_interp{point}, phipm); %still ion-photon, but with interpolation
            % now to swapped photon-photon state:

         nonmem = rhoML_rot{1}; % from the first  point of the same ion
    %    

        rho_swapped{point} = estimate_swapped_fidelity( nonmem, rho_interp{point}, 1, 'depolarize');  % Nonmem_Res.ML, fid_swapped(point)
        rho_swapped_plus = estimate_swapped_fidelity( rhoMC_rot{1}(:,:,sampl), rho_plus, 1, 'depolarize');
        rho_swapped_minus = estimate_swapped_fidelity( rhoMC_rot{1}(:,:,sampl), rho_minus, 1, 'depolarize');

        %     for  sampl = 1:out.samples
    %         rho_swappedMC{point, sampl} = estimate_swapped_fidelity(Nonmem_Res.MC(:,:,sampl), rho_interpMC{point}(:,:,sampl));
    %     end
        if(point > 1)
            for i = 1:4 
                rho_swapped_aver{point}{i} = rho_swapped_aver{point-1}{i}*sum(Prob(1:point-1))+rho_swapped{point}{i}*Prob(point); 
                rho_swapped_aver{point}{i} = rho_swapped_aver{point}{i}/sum(Prob(1:point));
                
                rho_swapped_aver_plus{point}{i} = rho_swapped_aver_plus{point-1}{i}*sum(Prob(1:point-1))+rho_swapped_plus{i}*Prob(point); 
                rho_swapped_aver_plus{point}{i} = rho_swapped_aver_plus{point}{i}/sum(Prob(1:point));
                
                rho_swapped_aver_minus{point}{i} = rho_swapped_aver_minus{point-1}{i}*sum(Prob(1:point-1))+rho_swapped_minus{i}*Prob(point); 
                rho_swapped_aver_minus{point}{i} = rho_swapped_aver_minus{point}{i}/sum(Prob(1:point));
    %             for  sampl = 1:out.samples
    %                 rho_swappedMC_aver{point,sampl}{i} = (rho_swappedMC_aver{point-1,sampl}{i}*(point-1)+rho_swappedMC{point,sampl}{i})/point;
    %             end
            end
        else
            rho_swapped_aver{point} = rho_swapped{point};
            rho_swapped_aver_plus{point} = rho_swapped_plus;
            rho_swapped_aver_minus{point} = rho_swapped_minus;
    %         for  sampl = 1:out.samples
    %             rho_swappedMC_aver{point,sampl} = rho_swappedMC{point, sampl};
    %         end
            if sampl == 1
                U = find_opt_U4(rho_swapped{point}, rho_base);
                Uav = U;%find_opt_U4(rho_swapped_aver{point}, rho_base);
            end;
        end
        %Uav = find_opt_U4(rho_swapped_aver{point}, rho_base); % can also    %optimize for each poit, but gives the same result

        for i = 1:4 % loop over the ions outcomes
           fid_swapped(i, point)  = mixedfid(U*rho_swapped{point}{i}*U', rho_base{i});
           %cumulative:
           fid_swapped_aver(i, point)  = mixedfid(Uav*rho_swapped_aver{point}{i}*Uav', rho_base{i});
        end
        fid_swapped_plus(point, sampl )  = mixedfid(U*rho_swapped_plus{i}*U', rho_base{i});
        fid_swapped_minus(point, sampl )  = mixedfid(U*rho_swapped_minus{i}*U', rho_base{i});
        fid_swapped_aver_plus(point, sampl )  = mixedfid(Uav*rho_swapped_aver_plus{point}{i}*Uav', rho_base{i});
        fid_swapped_aver_minus(point, sampl )  = mixedfid(Uav*rho_swapped_aver_minus{point}{i}*Uav', rho_base{i});
           
    end
    %fid_fine_err(point) = (mixedfid(rho_plus, phipm) - mixedfid(rho_minus, phipm))/2;
     fid_fine_err(point) = std([fid_iph_plus(point, :),fid_iph_minus(point, :)]);
     fid_swapped_err(point) = std([fid_swapped_plus(point, :),fid_swapped_minus(point, :)]);
     fid_swapped_aver_err(point) = std([fid_swapped_aver_plus(point, :),fid_swapped_aver_minus(point, :)]);
end
figure(fid_fig);
theor_color = 'r';
s = shadedErrorBar(X_fine*loop_duration, fid_fine, fid_fine_err);% '.', 'DisplayName', 'Interpolated fidelity with phi+ and rotation found for 0 mem steps. Test') % to check only
set(s.mainLine, 'Color', theor_color, 'LineWidth', 1.5, 'DisplayName', 'ion-photon model')
set( s.edge, 'LineStyle', 'none') 
set( s.patch, 'FaceColor', theor_color, 'FaceAlpha', 0.3)

s = shadedErrorBar(X_fine*loop_duration, fid_swapped(i, :), fid_swapped_err);% '.', 'DisplayName', 'Interpolated fidelity with phi+ and rotation found for 0 mem steps. Test') % to check only
set(s.mainLine, 'Color', 'Black', 'LineWidth', 1.5, 'LineStyle', '--', 'DisplayName', 'photon-photon model');
set( s.edge, 'LineStyle', 'none') 
set( s.patch, 'FaceColor',  'Black', 'FaceAlpha', 0.3)


legend('show')


